package ru.landgrafhomyak.db.serdha0.api.queries

import ru.landgrafhomyak.db.serdha0.api.misc.Column
import ru.landgrafhomyak.db.serdha0.api.misc.DatabaseType
import ru.landgrafhomyak.db.serdha0.api.misc.QueryParam
import ru.landgrafhomyak.db.serdha0.api.misc._Selectable
import ru.landgrafhomyak.db.serdha0.api.runtime.InputRow
import ru.landgrafhomyak.db.serdha0.api.table.RowId
import ru.landgrafhomyak.db.serdha0.api.table.Table

public interface DeleteQuery<qUE : Any> : _Query<qUE> {
	public interface SingleRow<qUE : Any> : DeleteQuery<qUE>, _Query.Params2Table<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any> : _Query.Constructor<SingleRow<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): SingleRow<qUE> =
				outQueries.deleteSingleRow(this)

			public val targetTable: Table<ttUE, *>

			public fun createDeleteSingleRowWithReturning(context: Scope<qUE, ttUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any> {
				public val tt: Table<ttUE, *>

				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> param(name: String, type: DT): QueryParam<RT, DT, qUE>

				public var rowToRemove: QueryParam<*, DatabaseType.ROW_ID<ttUE>, qUE>

				public fun <RT, DT : DatabaseType<RT>> returnRemovedValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, column: QueryParam<RT, DT, ttUE>): Column<RT, DT, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface FromSubquery<qUE : Any> : DeleteQuery<qUE>, _Query.Params2Table<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any, sUE : Any> : _Query.Constructor<FromSubquery<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): FromSubquery<qUE> =
				outQueries.deleteFromSubquery(this)

			public val targetTable: Table<ttUE, *>
			public val selector: _Selectable<sUE>

			public fun createDeleteFromSubqueryWithReturning(context: Scope<qUE, ttUE, sUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any, sUE : Any> {
				public val tt: Table<ttUE, *>
				public val sUExt: sUE

				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> param(name: String, type: DT): QueryParam<RT, DT, qUE>

				public val selectorParams: InputRow.WithRedirect<sUE, qUE>
				public var selectorColumn: Column<*, DatabaseType.ROW_ID<ttUE>, sUE>

				public fun <RT, DT : DatabaseType<RT>> returnFromSubquery(name: String, column: Column<RT, DT, sUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnRemovedValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, column: QueryParam<RT, DT, ttUE>): Column<RT, DT, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface MultipleRows<qUE : Any> : DeleteQuery<qUE>, _Query.Table2Void<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any> : _Query.Constructor<MultipleRows<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): MultipleRows<qUE> =
				outQueries.deleteMultipleRows(this)

			public val targetTable: Table<ttUE, *>

			public fun createMultipleRows(context: Scope<qUE, ttUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any> {
				public val tt: Table<ttUE, *>

				public val rowsToRemove: QueryParam<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowThrow()
			}
		}
	}
}